@@ -134,6 +134,42 @@ ICONFACTORY = TrytonIconFactory()
 ICONFACTORY.add_default()
 
 
+class ModelAccess(object):
+
+    batchnum = 100
+    _access = {}
+    _models = []
+
+    def load_models(self, refresh=False):
+        if not refresh:
+            self._access.clear()
+        del self._models[:]
+
+        try:
+            self._models = rpc.execute('model', 'ir.model', 'list_models',
+                rpc.CONTEXT)
+        except TrytonServerError:
+            pass
+
+    def __getitem__(self, model):
+        if model in self._access:
+            return self._access[model]
+        if model not in self._models:
+            self.load_models(refresh=True)
+        idx = self._models.index(model)
+        to_load = slice(max(0, idx - self.batchnum // 2),
+            idx + self.batchnum // 2)
+        try:
+            access = rpc.execute('model', 'ir.model.access', 'get_access',
+                self._models[to_load], rpc.CONTEXT)
+        except TrytonServerError:
+            access = {}
+        self._access.update(access)
+        return self._access[model]
+
+MODELACCESS = ModelAccess()
+
+
 def find_in_path(name):
     if os.name == "nt":
         sep = ';'
@@ -255,7 +291,8 @@ def request_server(server_widget):
 
 def get_toplevel_window():
     windows = [x for x in gtk.window_list_toplevels()
-        if x.window and x.props.visible]
+        if x.window and x.props.visible
+        and x.props.type == gtk.WINDOW_TOPLEVEL]
     trans2windows = dict((x.get_transient_for(), x) for x in windows)
     for window in set(windows) - set(trans2windows.iterkeys()):
         return window
@@ -827,7 +864,7 @@ class ErrorDialog(UniqueDialog):
     def __call__(self, title, details):
         if title == details:
             title = ''
-        log = logging.getLogger('common.message')
+        log = logging.getLogger(__name__)
         log.error(details + '\n' + title)
 
         response = super(ErrorDialog, self).__call__(title, details)
@@ -951,6 +988,8 @@ PLOCK = Lock()
 
 def process_exception(exception, *args, **kwargs):
 
+    rpc_execute = kwargs.get('rpc_execute', rpc.execute)
+
     if isinstance(exception, TrytonError):
         if exception.faultCode == 'BadFingerprint':
             warning(
@@ -976,14 +1015,15 @@ def process_exception(exception, *args, **kwargs):
                         'always': (res == 'always'),
                         }, rpc.CONTEXT)
                 try:
-                    rpc.execute(*args2)
+                    rpc_execute(*args2)
                 except TrytonServerError, exception:
                     process_exception(exception, *args2)
                 if args:
                     try:
-                        return rpc.execute(*args)
+                        return rpc_execute(*args)
                     except TrytonServerError, exception:
-                        return process_exception(exception, *args)
+                        return process_exception(exception, *args,
+                            rpc_execute=rpc_execute)
                 return True
             return False
         elif exception.faultCode == 'UserError':
@@ -996,9 +1036,10 @@ def process_exception(exception, *args, **kwargs):
                     if '_timestamp' in args[5]:
                         del args[5]['_timestamp']
                     try:
-                        return rpc.execute(*args)
+                        return rpc_execute(*args)
                     except TrytonServerError, exception:
-                        return process_exception(exception, *args)
+                        return process_exception(exception, *args,
+                            rpc_execute=rpc_execute)
                 return False
             else:
                 message(_('Concurrency Exception'), msg_type=gtk.MESSAGE_ERROR)
@@ -1026,9 +1067,10 @@ def process_exception(exception, *args, **kwargs):
                         continue
                     if args:
                         try:
-                            return rpc.execute(*args)
+                            return rpc_execute(*args)
                         except TrytonServerError, exception:
-                            return process_exception(exception, *args)
+                            return process_exception(exception, *args,
+                                rpc_execute=rpc_execute)
                     return True
             finally:
                 PLOCK.release()
@@ -1191,10 +1233,7 @@ class RPCProgress(object):
         thread.start_new_thread(self.start, ())
 
         watch = None
-        parent_sensitive = self.parent.props.sensitive
         i = 0
-        win = None
-        progressbar = None
         while (not self.res) and (not self.error):
             i += 1
             if i > 1:
@@ -1204,63 +1243,19 @@ class RPCProgress(object):
                     with gtk.gdk.lock:
                         while gtk.events_pending():
                             gtk.main_iteration()
-            if i > 10 and main_iteration_p:
-                if not win or not progressbar:
-                    win = gtk.Window(type=gtk.WINDOW_TOPLEVEL)
-                    win.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
-                    if hasattr(win, 'set_deletable'):
-                        win.set_deletable(False)
-                    win.set_decorated(False)
-                    vbox = gtk.VBox(False, 0)
-                    hbox = gtk.HBox(False, 13)
-                    hbox.set_border_width(10)
-                    img = gtk.Image()
-                    img.set_from_stock('tryton-dialog-information',
-                            gtk.ICON_SIZE_DIALOG)
-                    hbox.pack_start(img, expand=True, fill=False)
-                    vbox2 = gtk.VBox(False, 0)
-                    label = gtk.Label()
-                    label.set_markup(
-                        '<b>' + _('Operation in progress') + '</b>')
-                    label.set_alignment(0.0, 0.5)
-                    vbox2.pack_start(label, expand=True, fill=False)
-                    vbox2.pack_start(gtk.HSeparator(), expand=True, fill=True)
-                    vbox2.pack_start(gtk.Label(_("Please wait,\n" \
-                            "this operation may take a while...")),
-                            expand=True, fill=False)
-                    hbox.pack_start(vbox2, expand=True, fill=True)
-                    vbox.pack_start(hbox)
-                    progressbar = gtk.ProgressBar()
-                    progressbar.set_orientation(gtk.PROGRESS_LEFT_TO_RIGHT)
-                    vbox.pack_start(progressbar, expand=True, fill=False)
-                    viewport = gtk.Viewport()
-                    viewport.set_shadow_type(gtk.SHADOW_ETCHED_IN)
-                    viewport.add(vbox)
-                    win.add(viewport)
-                    win.set_transient_for(self.parent)
-                    win.set_modal(True)
-                    win.show_all()
-                    win.window.set_cursor(watch)
-                    self.parent.props.sensitive = False
-                with gtk.gdk.lock:
-                    progressbar.pulse()
             time.sleep(0.1)
-        self.parent.props.sensitive = parent_sensitive
         if self.parent.window:
             self.parent.window.set_cursor(None)
-        if win:
-            win.destroy()
-            with gtk.gdk.lock:
-                while gtk.events_pending():
-                    gtk.main_iteration()
         if self.exception:
             if process_exception_p:
-                if process_exception(self.exception):
-                    self.res = None
-                    self.error = False
-                    self.exception = None
-                    return self.run(process_exception_p)
-                raise RPCException(self.exception)
+                def rpc_execute(*args):
+                    return RPCProgress('execute',
+                        args).run(process_exception_p, main_iteration_p)
+                result = process_exception(self.exception, *self.args,
+                    rpc_execute=rpc_execute)
+                if result is False:
+                    raise RPCException(self.exception)
+                return result
             else:
                 raise self.exception
         return self.res
@@ -1308,8 +1303,6 @@ COLORS = {
     'required': '#d2d2ff',
 }
 
-HM_FORMAT = '%H:%M:%S'
-
 FLOAT_TIME_CONV = {
     'Y': 8760,
     'M': 672,
