@@ -9,9 +9,11 @@ from decimal import Decimal
 import datetime
 import time
 import io
+import collections
 
 from tryton.translate import date_format
-from tryton.common import HM_FORMAT, untimezoned_date, datetime_strftime
+from tryton.common import untimezoned_date, datetime_strftime
+from tryton.pyson import PYSONDecoder
 
 __all__ = ['DomainParser']
 
@@ -157,7 +159,7 @@ def append_ending_clause(domain, clause, deep):
 
 def default_operator(field):
     "Return default operator for field"
-    if field['type'] in ('char', 'text', 'many2one'):
+    if field['type'] in ('char', 'text', 'many2one', 'many2many', 'one2many'):
         return 'ilike'
     else:
         return '='
@@ -173,6 +175,10 @@ def negate_operator(operator):
         return 'not in'
 
 
+def time_format(field):
+    return PYSONDecoder({}).decode(field['format'])
+
+
 def convert_value(field, value):
     "Convert value for field"
 
@@ -211,7 +217,7 @@ def convert_value(field, value):
     def convert_datetime():
         try:
             return untimezoned_date(datetime.datetime(*time.strptime(value,
-                        date_format() + ' ' + HM_FORMAT)[:6]))
+                        date_format() + ' ' + time_format(field))[:6]))
         except ValueError:
             try:
                 return untimezoned_date(datetime.datetime(*time.strptime(value,
@@ -229,10 +235,16 @@ def convert_value(field, value):
 
     def convert_time():
         try:
-            return datetime.time(*time.strptime(value, HM_FORMAT)[3:6])
+            return datetime.time(*time.strptime(value,
+                    time_format(field))[3:6])
         except (ValueError, TypeError):
             return
 
+    def convert_many2one():
+        if value == '':
+            return None
+        return value
+
     converts = {
         'boolean': convert_boolean,
         'float': convert_float,
@@ -243,6 +255,7 @@ def convert_value(field, value):
         'datetime': convert_datetime,
         'date': convert_date,
         'time': convert_time,
+        'many2one': convert_many2one,
         }
     return converts.get(field['type'], lambda: value)()
 
@@ -327,6 +340,7 @@ def test_convert_selection():
 def test_convert_datetime():
     field = {
         'type': 'datetime',
+        'format': '"%H:%M:%S"',
         }
     for value, result in (
             ('12/04/2002', datetime.datetime(2002, 12, 4)),
@@ -352,6 +366,7 @@ def test_convert_date():
 def test_convert_time():
     field = {
         'type': 'time',
+        'format': '"%H:%M:%S"',
         }
     for value, result in (
             ('12:30:00', datetime.time(12, 30, 0)),
@@ -390,16 +405,26 @@ def format_value(field, value):
     def format_datetime():
         if not value:
             return ''
-        format_ = date_format() + ' ' + HM_FORMAT
+        format_ = date_format() + ' ' + time_format(field)
         if (not isinstance(value, datetime.datetime)
                 or value.time() == datetime.time.min):
             format_ = date_format()
         return datetime_strftime(value, format_)
 
+    def format_date():
+        if not value:
+            return ''
+        return datetime_strftime(value, date_format())
+
     def format_time():
         if not value:
             return ''
-        return datetime.time.strftime(value, HM_FORMAT)
+        return datetime.time.strftime(value, time_format(field))
+
+    def format_many2one():
+        if value is None:
+            return ''
+        return value
 
     converts = {
         'boolean': format_boolean,
@@ -409,8 +434,9 @@ def format_value(field, value):
         'selection': format_selection,
         'reference': format_selection,
         'datetime': format_datetime,
-        'date': format_datetime,
+        'date': format_date,
         'time': format_time,
+        'many2one': format_many2one,
         }
     if isinstance(value, (list, tuple)):
         return ';'.join(format_value(field, x) for x in value)
@@ -496,6 +522,7 @@ def test_format_selection():
 def test_format_datetime():
     field = {
         'type': 'datetime',
+        'format': '"%H:%M:%S"',
         }
     for value, result in (
             (datetime.date(2002, 12, 4), '12/04/2002'),
@@ -522,6 +549,7 @@ def test_format_date():
 def test_format_time():
     field = {
         'type': 'time',
+        'format': '"%H:%M:%S"',
         }
     for value, result in (
             (datetime.time(12, 30, 0), '"12:30:00"'),
@@ -553,7 +581,7 @@ def complete_value(field, value):
         yield datetime.date.today()
 
     def complete_time():
-        yield datetime.time()
+        yield datetime.datetime.now().time()
 
     completes = {
         'boolean': complete_boolean,
@@ -666,7 +694,11 @@ class DomainParser(object):
     "A parser for domain"
 
     def __init__(self, fields):
-        self.fields = dict((name, f)
+        if hasattr(collections, 'OrderedDict'):
+            odict = collections.OrderedDict
+        else:
+            odict = dict
+        self.fields = odict((name, f)
             for name, f in fields.iteritems()
             if f.get('searchable', True))
         self.strings = dict((f['string'].lower(), f)
