@@ -833,8 +833,8 @@ class Main(object):
     def sig_user_preferences(self, widget):
         if not self.close_pages():
             return False
-        win = Preference(rpc._USER)
-        if win.run():
+
+        def callback():
             rpc.context_reload()
             try:
                 prefs = RPCExecute('model', 'res.user', 'get_preferences',
@@ -856,8 +856,8 @@ class Main(object):
                     self.sig_win_menu()
                 CONFIG['client.lang'] = prefs['language']
             CONFIG.save()
-        self.sig_win_menu()
-        return True
+            self.sig_win_menu()
+        Preference(rpc._USER, callback)
 
     def sig_win_close(self, widget):
         self._sig_remove_book(widget,
@@ -929,6 +929,7 @@ class Main(object):
             except RPCException:
                 prefs = None
             common.ICONFACTORY.load_icons()
+            common.MODELACCESS.load_models()
             if prefs and 'language_direction' in prefs:
                 translate.set_language_direction(prefs['language_direction'])
                 CONFIG['client.language_direction'] = \
@@ -1189,6 +1190,8 @@ class Main(object):
     def _sig_remove_book(self, widget, page_widget):
         for page in self.pages:
             if page.widget == page_widget:
+                if not page.widget.props.sensitive:
+                    return
                 page_num = self.notebook.page_num(page.widget)
                 self.notebook.set_current_page(page_num)
                 res = page.sig_close()
@@ -1309,7 +1312,7 @@ class Main(object):
         self.refresh_ssl()
         common.message(_("Database dropped successfully!"))
 
-    def sig_db_restore(self, widget):
+    def sig_db_restore(self, widget=None):
         if not self.sig_logout(widget):
             return False
         filename = common.file_selection(_('Open Backup File to Restore...'),
@@ -1351,7 +1354,7 @@ class Main(object):
             else:
                 common.message(_('Database restore failed!'))
 
-    def sig_db_dump(self, widget):
+    def sig_db_dump(self, widget=None):
         if not self.sig_logout(widget):
             return False
         dialog = DBBackupDrop(function='backup')
