@@ -17,7 +17,6 @@ from tryton.common import (TRYTON_ICON, message, sur, sur_3b, COLOR_SCHEMES,
     timezoned_date)
 import tryton.common as common
 from tryton.translate import date_format
-from tryton.common import HM_FORMAT
 from tryton.common import RPCExecute, RPCException
 
 from tabcontent import TabContent
@@ -122,6 +121,14 @@ class Form(SignalEvent, TabContent):
             self.name = name
 
         self.create_tabcontent()
+
+        access = common.MODELACCESS[self.model]
+        for button, access_type in (
+                ('new', 'create'),
+                ('save', 'write'),
+                ):
+            self.buttons[button].props.sensitive = access[access_type]
+
         self.screen.signal_connect(self, 'record-message',
             self._record_message)
         self.screen.signal_connect(self, 'record-modified',
@@ -230,15 +237,11 @@ class Form(SignalEvent, TabContent):
         return self.screen.id_get()
 
     def sig_attach(self, widget=None):
-        obj_id = self.id_get()
-        if obj_id >= 0 and obj_id is not False:
-            def callback():
-                self.update_attachment_count(reload=True)
-            Attachment(self.model, obj_id, callback)
-        else:
-            self.message_info(_('No record selected!'))
-        self.update_attachment_count(reload=True)
-        return True
+        record_id = self.id_get()
+        if record_id is False or record_id < 0:
+            return
+        Attachment(self.model, record_id,
+            lambda: self.update_attachment_count(reload=True))
 
     def update_attachment_count(self, reload=False):
         record = self.screen.current_record
@@ -255,6 +258,9 @@ class Form(SignalEvent, TabContent):
             self.buttons['attach'].set_stock_id('tryton-attachment-hi')
         else:
             self.buttons['attach'].set_stock_id('tryton-attachment')
+        record_id = self.id_get()
+        self.buttons['attach'].props.sensitive = bool(
+            record_id >= 0 and record_id is not False)
 
     def sig_switch(self, widget=None):
         if not self.modified_save():
@@ -286,7 +292,7 @@ class Form(SignalEvent, TabContent):
                 value = str(line.get(key, False) or '/')
                 if line.get(key, False) \
                         and key in ('create_date', 'write_date'):
-                    display_format = date_format() + ' ' + HM_FORMAT
+                    display_format = date_format() + ' %H:%M:%S'
                     date = timezoned_date(line[key])
                     value = common.datetime_strftime(date, display_format)
                 message_str += val + ' ' + value + '\n'
@@ -295,6 +301,8 @@ class Form(SignalEvent, TabContent):
         return True
 
     def sig_remove(self, widget=None):
+        if not common.MODELACCESS[self.model]['delete']:
+            return
         if self.screen.current_view.view_type == 'form':
             msg = _('Are you sure to remove this record?')
         else:
@@ -311,8 +319,7 @@ class Form(SignalEvent, TabContent):
         fields = {}
         for name, field in self.screen.group.fields.iteritems():
             fields[name] = field.attrs
-        win = WinImport(self.model)
-        win.run()
+        WinImport(self.model)
 
     def sig_save_as(self, widget=None):
         while self.screen.view_to_load:
@@ -320,11 +327,11 @@ class Form(SignalEvent, TabContent):
         fields = {}
         for name, field in self.screen.group.fields.iteritems():
             fields[name] = field.attrs
-        win = WinExport(self.model, self.ids_get(),
-                context=self.context)
-        win.run()
+        WinExport(self.model, self.ids_get(), context=self.context)
 
     def sig_new(self, widget=None, autosave=True):
+        if not common.MODELACCESS[self.model]['create']:
+            return
         if autosave:
             if not self.modified_save():
                 return
@@ -333,6 +340,8 @@ class Form(SignalEvent, TabContent):
         self.activate_save()
 
     def sig_copy(self, widget=None):
+        if not common.MODELACCESS[self.model]['create']:
+            return
         if not self.modified_save():
             return
         res_ids = self.sel_ids_get()
@@ -346,6 +355,8 @@ class Form(SignalEvent, TabContent):
             'green')
 
     def sig_save(self, widget=None):
+        if not common.MODELACCESS[self.model]['write']:
+            return
         if self.screen.save_current():
             self.message_info(_('Record saved!'), 'green')
             return True
@@ -395,19 +406,24 @@ class Form(SignalEvent, TabContent):
         return True
 
     def sig_action(self, widget):
-        self.buttons['action'].props.active = True
+        if self.buttons['action'].props.sensitive:
+            self.buttons['action'].props.active = True
 
     def sig_print(self, widget):
-        self.buttons['print'].props.active = True
+        if self.buttons['print'].props.sensitive:
+            self.buttons['print'].props.active = True
 
     def sig_print_open(self, widget):
-        self.buttons['open'].props.active = True
+        if self.buttons['open'].props.sensitive:
+            self.buttons['open'].props.active = True
 
     def sig_print_email(self, widget):
-        self.buttons['email'].props.active = True
+        if self.buttons['email'].props.sensitive:
+            self.buttons['email'].props.active = True
 
     def sig_relate(self, widget):
-        self.buttons['relate'].props.active = True
+        if self.buttons['relate'].props.sensitive:
+            self.buttons['relate'].props.active = True
 
     def sig_search(self, widget):
         search_container = self.screen.screen_container
@@ -449,11 +465,13 @@ class Form(SignalEvent, TabContent):
         if signal_data[0]:
             name = str(signal_data[0])
         for button_id in ('print', 'action', 'relate', 'email', 'open', 'save',
-                'switch', 'attach'):
+                'attach'):
             button = self.buttons[button_id]
             can_be_sensitive = getattr(button, '_can_be_sensitive', True)
             button.props.sensitive = (bool(signal_data[0])
                 and can_be_sensitive)
+        button_switch = self.buttons['switch']
+        button_switch.props.sensitive = self.screen.number_of_views > 1
 
         msg = name + ' / ' + str(signal_data[1])
         if signal_data[1] < signal_data[2]:
@@ -470,6 +488,7 @@ class Form(SignalEvent, TabContent):
 
     def _record_saved(self, screen, signal_data):
         self.activate_save()
+        self.update_attachment_count()
 
     def modified_save(self):
         self.screen.current_view.set_value()
@@ -501,7 +520,7 @@ class Form(SignalEvent, TabContent):
             'id': record_id,
             'ids': record_ids,
         }
-        Action._exec_action(action, data, {})
+        Action._exec_action(action, data, self.context)
 
     def activate_save(self):
         self.buttons['save'].props.sensitive = self.screen.modified()
@@ -578,4 +597,4 @@ class Form(SignalEvent, TabContent):
 
     def set_cursor(self):
         if self.screen:
-            self.screen.set_cursor()
+            self.screen.set_cursor(reset_view=False)
