@@ -3,7 +3,7 @@
 import os
 import tempfile
 import locale
-from tryton.common import datetime_strftime, HM_FORMAT, \
+from tryton.common import datetime_strftime, \
         domain_inversion, eval_domain, localize_domain, unlocalize_domain, \
         merge, inverse_leaf, EvalEnvironment
 import tryton.common as common
@@ -214,7 +214,7 @@ class DateTimeField(CharField):
         if not isinstance(value, datetime.datetime):
             try:
                 value = datetime.datetime(*time.strptime(value,
-                        date_format() + ' ' + HM_FORMAT)[:6])
+                        date_format() + ' ' + self.time_format(record))[:6])
                 value = common.untimezoned_date(value)
             except ValueError:
                 value = self._default
@@ -225,9 +225,13 @@ class DateTimeField(CharField):
         value = super(DateTimeField, self).get_client(record)
         if value:
             value = common.timezoned_date(value)
-            return datetime_strftime(value, date_format() + ' ' + HM_FORMAT)
+            return datetime_strftime(value, date_format() + ' ' +
+                self.time_format(record))
         return ''
 
+    def time_format(self, record):
+        return record.expr_eval(self.attrs['format'])
+
 
 class DateField(CharField):
 
@@ -255,7 +259,8 @@ class TimeField(CharField):
     def set_client(self, record, value, force_change=False):
         if not isinstance(value, datetime.time):
             try:
-                value = datetime.time(*time.strptime(value, HM_FORMAT)[3:6])
+                value = datetime.time(*time.strptime(value,
+                        self.time_format(record))[3:6])
             except ValueError:
                 value = None
         super(TimeField, self).set_client(record, value,
@@ -263,10 +268,13 @@ class TimeField(CharField):
 
     def get_client(self, record):
         value = super(TimeField, self).get_client(record)
-        if value:
-            return value.strftime(HM_FORMAT)
+        if value is not None:
+            return value.strftime(self.time_format(record))
         return ''
 
+    def time_format(self, record):
+        return record.expr_eval(self.attrs['format'])
+
 
 class NumberField(CharField):
     _default = None
@@ -389,7 +397,12 @@ class M2OField(CharField):
     def set_client(self, record, value, force_change=False):
         if isinstance(value, (tuple, list)):
             value, rec_name = value
-            record.value[self.name + '.rec_name'] = rec_name
+        else:
+            if value == self.get(record):
+                rec_name = record.value.get(self.name + '.rec_name', '')
+            else:
+                rec_name = ''
+        record.value[self.name + '.rec_name'] = rec_name
         super(M2OField, self).set_client(record, value,
             force_change=force_change)
 
@@ -407,8 +420,8 @@ class M2OField(CharField):
                 value = None
         if not rec_name and value >= 0:
             try:
-                result = RPCExecute('model', self.attrs['relation'], 'read',
-                    value, ['rec_name'])
+                result, = RPCExecute('model', self.attrs['relation'], 'read',
+                    [value], ['rec_name'])
             except RPCException:
                 return False
             rec_name = result['rec_name'] or ''
@@ -530,7 +543,7 @@ class O2MField(CharField):
                     get_readonly=readonly, get_modifiedonly=modified)
                 values.pop(parent_name, None)
                 if record2.modified and values:
-                    result.append(('write', record2.id, values))
+                    result.append(('write', [record2.id], values))
                 result[0][1].append(record2.id)
             else:
                 values = record2.get(check_load=check_load,
@@ -569,7 +582,8 @@ class O2MField(CharField):
             return []
         for record2 in record.value[self.name]:
             if not (record2.deleted or record2.removed):
-                result.append(record2.get_eval(check_load=check_load))
+                result.append(
+                    record2.get_on_change_value(check_load=check_load))
         return result
 
     def set(self, record, value, modified=False):
@@ -664,7 +678,10 @@ class O2MField(CharField):
     def set_on_change(self, record, value):
         self._set_default_value(record)
         if isinstance(value, (list, tuple)):
-            return self.set(record, value, modified=False)
+            self.set(record, value, modified=False)
+            record.modified_fields.setdefault(self.name)
+            record.signal('record-modified')
+            return True
 
         if value and (value.get('add') or value.get('update')):
             context = self.context_get(record)
@@ -731,10 +748,8 @@ class O2MField(CharField):
     def state_set(self, record, states=('readonly', 'required', 'invisible')):
         self._set_default_value(record)
         super(O2MField, self).state_set(record, states=states)
-        if self.get_state_attrs(record).get('readonly', False):
-            record.value[self.name].readonly = True
-        else:
-            record.value[self.name].readonly = False
+        record.value[self.name].readonly = self.get_state_attrs(record).get(
+            'readonly', False)
 
     def get_removed_ids(self, record):
         return [x.id for x in record.value[self.name].record_removed]
@@ -780,6 +795,9 @@ class M2MField(O2MField):
             record.signal('record-modified')
             record.signal('record-changed')
 
+    def get_on_change_value(self, record, check_load=True):
+        return self.get_eval(record, check_load=check_load)
+
 
 class ReferenceField(CharField):
 
@@ -794,16 +812,25 @@ class ReferenceField(CharField):
             return None
 
     def get(self, record, check_load=True, readonly=True, modified=False):
-        if record.value.get(self.name):
+        if (record.value.get(self.name)
+                and record.value[self.name][0]
+                and record.value[self.name][1] >= 0):
             return ','.join(map(str, record.value[self.name]))
         return None
 
     def set_client(self, record, value, force_change=False):
         if value:
+            if isinstance(value, basestring):
+                value = value.split(',')
             ref_model, ref_id = value
             if isinstance(ref_id, (tuple, list)):
                 ref_id, rec_name = ref_id
-                record.value[self.name + '.rec_name'] = rec_name
+            else:
+                if '%s,%s' % (ref_model, ref_id) == self.get(record):
+                    rec_name = record.value.get(self.name + '.rec_name', '')
+                else:
+                    rec_name = ''
+            record.value[self.name + '.rec_name'] = rec_name
             value = (ref_model, ref_id)
         super(ReferenceField, self).set_client(record, value,
             force_change=force_change)
@@ -827,7 +854,7 @@ class ReferenceField(CharField):
         if ref_model and ref_id >= 0:
             if not rec_name and ref_id >= 0:
                 try:
-                    result = RPCExecute('model', ref_model, 'read', ref_id,
+                    result, = RPCExecute('model', ref_model, 'read', [ref_id],
                         ['rec_name'])
                 except RPCException:
                     return
@@ -885,8 +912,8 @@ class BinaryField(CharField):
                 return ''
             context = record.context_get()
             try:
-                values = RPCExecute('model', record.model_name, 'read',
-                    record.id, [self.name], context=context)
+                values, = RPCExecute('model', record.model_name, 'read',
+                    [record.id], [self.name], context=context)
             except RPCException:
                 return ''
             _, filename = tempfile.mkstemp(prefix='tryton_')
