@@ -1,10 +1,15 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
+try:
+    from weakref import WeakSet
+except ImportError:
+    from weakrefset import WeakSet
+
 from record import Record
 from field import Field, O2MField
 from tryton.signal_event import SignalEvent
 from tryton.common.domain_inversion import is_leaf
-from tryton.common import RPCExecute, RPCException
+from tryton.common import RPCExecute, RPCException, MODELACCESS
 
 
 class Group(SignalEvent, list):
@@ -30,12 +35,25 @@ class Group(SignalEvent, list):
         self.load(ids)
         self.record_deleted, self.record_removed = [], []
         self.on_write = set()
-        self.readonly = readonly
-        if self._context.get('_datetime'):
-            self.readonly = True
+        self.__readonly = readonly
         self.__id2record = {}
         self.__field_childs = None
         self.exclude_field = None
+        self.pool = WeakSet()
+        self.skip_model_access = False
+
+    @property
+    def readonly(self):
+        # Must skip res.user for Preference windows
+        if (self._context.get('_datetime')
+                or (not MODELACCESS[self.model_name]['write']
+                    and not self.skip_model_access)):
+            return True
+        return self.__readonly
+
+    @readonly.setter
+    def readonly(self, value):
+        self.__readonly = value
 
     @property
     def domain(self):
@@ -231,6 +249,9 @@ class Group(SignalEvent, list):
         ctx = self._context.copy()
         if self.parent:
             ctx.update(self.parent.context_get())
+            if self.child_name in self.parent.group.fields:
+                field = self.parent.group.fields[self.child_name]
+                ctx.update(field.context_get(self.parent, check_load=False))
         ctx.update(self._context)
         if self.parent_datetime_field:
             ctx['_datetime'] = self.parent.get_eval(check_load=False
