@@ -1,10 +1,5 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-try:
-    from weakref import WeakSet
-except ImportError:
-    from weakrefset import WeakSet
-from collections import defaultdict
 import tryton.rpc as rpc
 from tryton.signal_event import SignalEvent
 import tryton.common as common
@@ -13,8 +8,6 @@ import field as fields
 from functools import reduce
 from tryton.common import RPCExecute, RPCException
 
-POOL = defaultdict(WeakSet)
-
 
 class Record(SignalEvent):
 
@@ -42,7 +35,7 @@ class Record(SignalEvent):
         self.autocompletion = {}
         self.exception = False
         self.destroyed = False
-        POOL[model_name].add(self)
+        self.pool.add(self)
 
     def __getitem__(self, name):
         if name not in self._loaded and self.id >= 0:
@@ -74,7 +67,7 @@ class Record(SignalEvent):
                     n += 1
             record_context = self.context_get()
             if loading == 'eager' and len(id2record) < 80:
-                for record in POOL[self.model_name]:
+                for record in self.pool:
                     if (name not in record._loaded
                             and record.id >= 0
                             and record.id not in id2record
@@ -131,6 +124,14 @@ class Record(SignalEvent):
     def modified(self):
         return bool(self.modified_fields)
 
+    @property
+    def pool(self):
+        group = self.group
+        while (group.parent is not None
+                and group.parent.model_name == self.model_name):
+            group = group.parent
+        return group.pool
+
     @property
     def parent(self):
         return self.group.parent
@@ -139,6 +140,15 @@ class Record(SignalEvent):
     def parent_name(self):
         return self.group.parent_name
 
+    @property
+    def depth(self):
+        parent = self.parent
+        i = 0
+        while parent:
+            i += 1
+            parent = parent.parent
+        return i
+
     def set_modified(self, value):
         if value:
             self.signal('record-modified')
@@ -263,6 +273,15 @@ class Record(SignalEvent):
             result.update(field.get_timestamp(self))
         return result
 
+    def pre_validate(self):
+        values = self.get()
+        try:
+            RPCExecute('model', self.model_name, 'pre_validate', values,
+                context=self.context_get())
+        except RPCException:
+            return False
+        return True
+
     def save(self, force_reload=True):
         if self.id < 0 or self.modified:
             if self.id < 0:
@@ -284,9 +303,8 @@ class Record(SignalEvent):
                     context = context.copy()
                     context['_timestamp'] = self.get_timestamp()
                     try:
-                        if not RPCExecute('model', self.model_name, 'write',
-                                [self.id], value, context=context):
-                            return False
+                        RPCExecute('model', self.model_name, 'write',
+                            [self.id], value, context=context)
                     except RPCException:
                         return False
             self._loaded.clear()
@@ -335,10 +353,14 @@ class Record(SignalEvent):
             except RPCException:
                 return
             if (self.parent
-                    and self.parent_name in self.group.fields
-                    and (self.group.fields[self.parent_name].attrs['relation']
-                        == self.group.parent.model_name)):
-                vals[self.parent_name] = self.parent.id
+                    and self.parent_name in self.group.fields):
+                parent_field = self.group.fields[self.parent_name]
+                if isinstance(parent_field, fields.ReferenceField):
+                    vals[self.parent_name] = (
+                        self.parent.model_name, self.parent.id)
+                elif (self.group.fields[self.parent_name].attrs['relation']
+                        == self.group.parent.model_name):
+                    vals[self.parent_name] = self.parent.id
             self.set_default(vals)
         for fieldname, fieldinfo in self.group.fields.iteritems():
             if not fieldinfo.attrs.get('autocomplete'):
@@ -347,8 +369,8 @@ class Record(SignalEvent):
 
     def rec_name(self):
         try:
-            return RPCExecute('model', self.model_name, 'read', self.id,
-                ['rec_name'], context=self.context_get())['rec_name']
+            return RPCExecute('model', self.model_name, 'read', [self.id],
+                ['rec_name'], context=self.context_get())[0]['rec_name']
         except RPCException:
             return ''
 
@@ -447,10 +469,11 @@ class Record(SignalEvent):
             self._check_load()
         ctx = rpc.CONTEXT.copy()
         ctx['context'] = ctx.copy()
+        ctx['context'].update(self.context_get())
         for name, field in self.group.fields.items():
             ctx[name] = field.get_eval(self, check_load=check_load)
 
-        ctx['context'].update(self.context_get())
+        ctx['active_model'] = self.model_name
         ctx['active_id'] = self.id
         ctx['id'] = self.id  # Force local id
         ctx['_user'] = rpc._USER
@@ -527,12 +550,14 @@ class Record(SignalEvent):
         if fieldnames:
             try:
                 result = RPCExecute('model', self.model_name, 'on_change_with',
-                    list(fieldnames), values, context=self.context_get())
+                    values, list(fieldnames), context=self.context_get())
             except RPCException:
                 return
             for fieldname, value in result.items():
                 self.group.fields[fieldname].set_on_change(self, value)
         for fieldname in later:
+            on_change_with = self.group.fields[fieldname].attrs.get(
+                    'on_change_with')
             values = self._get_on_change_args(on_change_with)
             try:
                 result = RPCExecute('model', self.model_name,
@@ -576,6 +601,8 @@ class Record(SignalEvent):
         return self.attachment_count
 
     def destroy(self):
+        # Get reference to pool before unref group
+        pool = self.pool
         for v in self.value.itervalues():
             if hasattr(v, 'destroy'):
                 v.destroy()
@@ -584,4 +611,4 @@ class Record(SignalEvent):
         self.value = None
         self.next = None
         self.destroyed = True
-        POOL[self.model_name].remove(self)
+        pool.remove(self)
