@@ -17,12 +17,18 @@ from tryton.config import CONFIG
 from tryton.exceptions import TrytonServerError, TrytonServerUnavailable
 from tryton.jsonrpc import JSONEncoder
 from tryton.common.domain_parser import DomainParser
-from tryton.common import RPCExecute, RPCException
+from tryton.common import RPCExecute, RPCException, MODELACCESS, \
+    node_attributes, sur
+from tryton.action import Action
 
 
 class Screen(SignalEvent):
     "Screen"
 
+    # Width of tree columns per model
+    # It is shared with all connection but it is the price for speed.
+    tree_column_width = collections.defaultdict(lambda: {})
+
     def __init__(self, model_name, view_ids=None, mode=None, context=None,
             views_preload=None, domain=None, row_activate=None, limit=None,
             readonly=False, exclude_field=None, sort=None, search_value=None,
@@ -43,6 +49,8 @@ class Screen(SignalEvent):
         super(Screen, self).__init__()
 
         self.readonly = readonly
+        if not MODELACCESS[model_name]['write']:
+            self.readonly = True
         self.search_count = 0
         if not row_activate:
             self.row_activate = self.default_row_activate
@@ -73,6 +81,7 @@ class Screen(SignalEvent):
         self.expanded_nodes = collections.defaultdict(
             lambda: collections.defaultdict(lambda: None))
         self.domain_parser = None
+        self.pre_validate = False
 
         if mode:
             self.view_to_load = mode[1:]
@@ -104,8 +113,19 @@ class Screen(SignalEvent):
                     continue
                 props['selection'] = self.get_selection(props)
 
-            self.domain_parser = DomainParser(
-                self.fields_view_tree['fields'])
+            # Filter only fields in XML view
+            xml_dom = xml.dom.minidom.parseString(
+                self.fields_view_tree['arch'])
+            root_node, = xml_dom.childNodes
+            xml_fields = [node_attributes(node).get('name')
+                for node in root_node.childNodes if node.nodeName == 'field']
+            if hasattr(collections, 'OrderedDict'):
+                odict = collections.OrderedDict
+            else:
+                odict = dict
+            fields = odict((name, fields[name]) for name in  xml_fields)
+
+            self.domain_parser = DomainParser(fields)
 
             self.screen_container.set_screen(self)
             self.screen_container.show_filter()
@@ -288,6 +308,10 @@ class Screen(SignalEvent):
             self.switch_view(view_type='form')
             return True
 
+    @property
+    def number_of_views(self):
+        return len(self.views) + len(self.view_to_load)
+
     def switch_view(self, view_type=None, default=True, context=None):
         if not self.parent and self.modified():
             return
@@ -302,7 +326,7 @@ class Screen(SignalEvent):
             self.current_view.display()
             return
         if not view_type or self.current_view.view_type != view_type:
-            for i in xrange(len(self.views) + len(self.view_to_load)):
+            for i in xrange(self.number_of_views):
                 if len(self.view_to_load):
                     self.load_view_to_load()
                     self.__current_view = len(self.views) - 1
@@ -351,12 +375,12 @@ class Screen(SignalEvent):
                 self.fields_view_tree = view
             break
 
-        # Ensure that loading is always eager for fields on tree view
-        # and always lazy for fields only on form view
-        if node.localName == 'tree':
-            loading = 'eager'
-        else:
+        # Ensure that loading is always lazy for fields on form view
+        # and always eager for fields on tree or graph view
+        if node.localName == 'form':
             loading = 'lazy'
+        else:
+            loading = 'eager'
         for field in fields:
             if field not in self.group.fields or loading == 'eager':
                 fields[field]['loading'] = loading
@@ -394,8 +418,6 @@ class Screen(SignalEvent):
         return False
 
     def new(self, default=True, context=None):
-        if self.group.readonly:
-            return
         if context is None:
             context = {}
         if self.current_view and \
@@ -518,8 +540,7 @@ class Screen(SignalEvent):
             return True
         return False
 
-    def reload(self, written=False):
-        ids = self.sel_ids_get()
+    def reload(self, ids, written=False):
         self.group.reload(ids)
         if written:
             self.group.written(ids)
@@ -542,6 +563,8 @@ class Screen(SignalEvent):
         if not records:
             return
         if delete:
+            # Must delete children records before parent
+            records.sort(key=lambda r: r.depth, reverse=True)
             if not self.group.delete(records):
                 return False
 
@@ -652,6 +675,14 @@ class Screen(SignalEvent):
     def display(self, res_id=None, set_cursor=False):
         if res_id:
             self.current_record = self.group.get(res_id)
+        else:
+            if (self.current_record
+                    and self.current_record in self.current_record.group):
+                pass
+            elif self.group:
+                self.current_record = self.group[0]
+            else:
+                self.current_record = None
         if self.views:
             #XXX To remove when calendar will be implemented
             if self.current_view.view_type == 'calendar' and \
@@ -668,6 +699,8 @@ class Screen(SignalEvent):
             if set_cursor:
                 self.set_cursor(reset_view=False)
         self.set_tree_state()
+        # Force record-message signal
+        self.current_record = self.current_record
 
     def display_next(self):
         view = self.current_view
@@ -680,7 +713,9 @@ class Screen(SignalEvent):
             store = view.store
             iter_ = store.get_iter(end)
             self.current_record = store.get_value(iter_, 0)
-        elif view.view_type == 'form' and self.current_record.group:
+        elif (view.view_type == 'form'
+                and self.current_record
+                and self.current_record.group):
             group = self.current_record.group
             record = self.current_record
             while group:
@@ -707,7 +742,7 @@ class Screen(SignalEvent):
                 break
             self.current_record = record
         else:
-            self.current_record = len(self.group) and self.group[0]
+            self.current_record = self.group[0] if len(self.group) else None
         self.set_cursor(reset_view=False)
         view.display()
 
@@ -722,7 +757,9 @@ class Screen(SignalEvent):
             store = view.store
             iter_ = store.get_iter(start)
             self.current_record = store.get_value(iter_, 0)
-        elif view.view_type == 'form' and self.current_record.group:
+        elif (view.view_type == 'form'
+                and self.current_record
+                and self.current_record.group):
             group = self.current_record.group
             record = self.current_record
             idx = group.index(record) - 1
@@ -739,7 +776,7 @@ class Screen(SignalEvent):
                     record = parent
             self.current_record = record
         else:
-            self.current_record = len(self.group) and self.group[-1]
+            self.current_record = self.group[-1] if len(self.group) else None
         self.set_cursor(reset_view=False)
         view.display()
 
@@ -766,3 +803,23 @@ class Screen(SignalEvent):
         if self.model_name == 'res.request':
             from tryton.gui.main import Main
             Main.get_main().request_set()
+
+    def button(self, button):
+        'Execute button on the current record'
+        if button.get('confirm', False) and not sur(button['confirm']):
+            return
+        record = self.current_record
+        record.save(force_reload=False)
+        context = record.context_get()
+        try:
+            action_id = RPCExecute('model', self.model_name, button['name'],
+                [record.id], context=context)
+        except RPCException:
+            action_id = None
+        if action_id:
+            Action.execute(action_id, {
+                    'model': self.model_name,
+                    'id': record.id,
+                    'ids': [record.id],
+                    }, context=context)
+        self.reload([record.id], written=True)
