@@ -48,16 +48,19 @@ class Image(WidgetInterface):
             self.but_add.set_image(img_add)
             self.but_add.set_relief(gtk.RELIEF_NONE)
             self.but_add.connect('clicked', self.sig_add)
+            self.but_add.connect('enter-notify-event', self.enter)
             self.tooltips.set_tip(self.but_add, _('Select an Image...'))
             hbox.pack_start(self.but_add, expand=False, fill=False)
 
             if self.filename:
                 self.but_open = gtk.Button()
                 img_open = gtk.Image()
-                img_open.set_from_stock('tryton-open', gtk.ICON_SIZE_SMALL_TOOLBAR)
+                img_open.set_from_stock('tryton-open',
+                    gtk.ICON_SIZE_SMALL_TOOLBAR)
                 self.but_open.set_image(img_open)
                 self.but_open.set_relief(gtk.RELIEF_NONE)
                 self.but_open.connect('clicked', self.sig_open)
+                self.but_open.connect('enter-notify-event', self.enter)
                 self.tooltips.set_tip(self.but_open, _('Open...'))
                 hbox.pack_start(self.but_open, expand=False, fill=False)
             else:
@@ -65,19 +68,23 @@ class Image(WidgetInterface):
 
             self.but_save_as = gtk.Button()
             img_save_as = gtk.Image()
-            img_save_as.set_from_stock('tryton-save', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            img_save_as.set_from_stock('tryton-save',
+                gtk.ICON_SIZE_SMALL_TOOLBAR)
             self.but_save_as.set_image(img_save_as)
             self.but_save_as.set_relief(gtk.RELIEF_NONE)
             self.but_save_as.connect('clicked', self.sig_save_as)
+            self.but_save_as.connect('enter-notify-event', self.enter)
             self.tooltips.set_tip(self.but_save_as, _('Save As...'))
             hbox.pack_start(self.but_save_as, expand=False, fill=False)
 
             self.but_remove = gtk.Button()
             img_remove = gtk.Image()
-            img_remove.set_from_stock('tryton-clear', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            img_remove.set_from_stock('tryton-clear',
+                gtk.ICON_SIZE_SMALL_TOOLBAR)
             self.but_remove.set_image(img_remove)
             self.but_remove.set_relief(gtk.RELIEF_NONE)
             self.but_remove.connect('clicked', self.sig_remove)
+            self.but_remove.connect('enter-notify-event', self.enter)
             self.tooltips.set_tip(self.but_remove, _('Clear'))
             hbox.pack_start(self.but_remove, expand=False, fill=False)
 
@@ -193,7 +200,7 @@ class Image(WidgetInterface):
         if self.field:
             value = self.field.get_client(self.record)
         if isinstance(value, (int, long)):
-            if value > 10**6:
+            if value > 10 ** 6:
                 value = False
             else:
                 value = self.field.get_data(self.record)
@@ -207,8 +214,8 @@ class Image(WidgetInterface):
             try:
                 loader = gtk.gdk.PixbufLoader(ftype)
                 loader.write(data, len(data))
-                pixbuf = loader.get_pixbuf()
                 loader.close()
+                pixbuf = loader.get_pixbuf()
             except glib.GError:
                 continue
             if pixbuf:
@@ -216,8 +223,8 @@ class Image(WidgetInterface):
         if not pixbuf:
             loader = gtk.gdk.PixbufLoader('png')
             loader.write(NOIMAGE, len(NOIMAGE))
-            pixbuf = loader.get_pixbuf()
             loader.close()
+            pixbuf = loader.get_pixbuf()
 
         img_height = pixbuf.get_height()
         if img_height > self.height:
