@@ -164,6 +164,10 @@ class WidgetInterface(object):
     def cancel(self):
         pass
 
+    def enter(self, widget, event):
+        if self.view:
+            self.view.set_value(focused_widget=True)
+
 
 class TranslateDialog(NoModal):
 
@@ -207,17 +211,17 @@ class TranslateDialog(NoModal):
                 )
             try:
                 value = RPCExecute('model', self.widget.record.model_name,
-                    'read', self.widget.record.id, [self.widget.field_name],
+                    'read', [self.widget.record.id], [self.widget.field_name],
                     context={'language': language['code']}
-                    )[self.widget.field_name]
+                    )[0][self.widget.field_name]
             except RPCException:
                 return
             context['fuzzy_translation'] = True
             try:
                 fuzzy_value = RPCExecute('model',
                     self.widget.record.model_name, 'read',
-                    self.widget.record.id, [self.widget.field_name],
-                    context=context)[self.widget.field_name]
+                    [self.widget.record.id], [self.widget.field_name],
+                    context=context)[0][self.widget.field_name]
             except RPCException:
                 return
             widget = self.widget.translate_widget()
@@ -272,7 +276,7 @@ class TranslateDialog(NoModal):
                     )
                 try:
                     RPCExecute('model', self.widget.record.model_name, 'write',
-                        self.widget.record.id, {
+                        [self.widget.record.id], {
                             self.widget.field_name: value,
                             }, context=context)
                 except RPCException:
@@ -295,6 +299,7 @@ class TranslateMixin:
         button.set_image(img)
         button.set_relief(gtk.RELIEF_NONE)
         button.connect('clicked', self.translate)
+        button.connect('enter-notify-event', self.enter)
         return button
 
     def translate(self, widget):
