@@ -27,7 +27,7 @@ class Many2One(WidgetInterface):
         self.wid_text.set_property('width-chars', 13)
         self.wid_text.set_property('activates_default', True)
         self.wid_text.connect('key-press-event', self.send_modified)
-        self.wid_text.connect_after('key_press_event', self.sig_key_press)
+        self.wid_text.connect('key_press_event', self.sig_key_press)
         self.wid_text.connect('populate-popup', self._populate_popup)
         self.wid_text.connect('focus-in-event', lambda x, y: self._focus_in())
         self.wid_text.connect('focus-out-event',
@@ -44,6 +44,7 @@ class Many2One(WidgetInterface):
         self.but_open.set_image(img_find)
         self.but_open.set_relief(gtk.RELIEF_NONE)
         self.but_open.connect('clicked', self.sig_edit)
+        self.but_open.connect('enter-notify-event', self.enter)
         self.but_open.set_alignment(0.5, 0.5)
 
         self.but_new = gtk.Button()
@@ -52,6 +53,7 @@ class Many2One(WidgetInterface):
         self.but_new.set_image(img_new)
         self.but_new.set_relief(gtk.RELIEF_NONE)
         self.but_new.connect('clicked', self.sig_new)
+        self.but_new.connect('enter-notify-event', self.enter)
         self.but_new.set_alignment(0.5, 0.5)
 
         self.widget.pack_end(self.but_new, expand=False, fill=False)
@@ -75,13 +77,29 @@ class Many2One(WidgetInterface):
 
     def _readonly_set(self, value):
         self._readonly = value
-        self.wid_text.set_editable(not value)
-        self.but_new.set_sensitive(not value)
+        self._set_button_sensitive()
         if value:
             self.widget.set_focus_chain([])
         else:
             self.widget.set_focus_chain([self.wid_text])
 
+    def _set_button_sensitive(self):
+        model = self.get_model()
+        if model:
+            access = common.MODELACCESS[model]
+        else:
+            access = {
+                'create': True,
+                'read': True,
+                }
+        self.wid_text.set_editable(not self._readonly)
+        self.but_new.set_sensitive(bool(
+                not self._readonly
+                and self.attrs.get('create', True)
+                and access['create']))
+        self.but_open.set_sensitive(bool(
+                access['read']))
+
     @property
     def modified(self):
         if self.record and self.field:
@@ -106,7 +124,10 @@ class Many2One(WidgetInterface):
     def id_from_value(value):
         return value
 
-    def sig_activate(self, widget, event=None, key_press=False):
+    def sig_activate(self, widget=None, event=None, key_press=False):
+        model = self.get_model()
+        if not model or not common.MODELACCESS[model]['read']:
+            return
         if not self.focus_out or not self.field:
             return
         self.changed = False
@@ -115,7 +136,7 @@ class Many2One(WidgetInterface):
 
         self.focus_out = False
         if model and not self.has_target(value):
-            if not key_press and not event:
+            if not key_press and not event and widget:
                 widget.emit_stop_by_name('activate')
             if (not self._readonly
                     and (self.wid_text.get_text()
@@ -143,7 +164,6 @@ class Many2One(WidgetInterface):
                     self.field.set_client(self.record,
                         self.value_from_id(ids[0]), force_change=True)
                     self.focus_out = True
-                    self.display(self.record, self.field)
                     self.changed = True
                     return
 
@@ -151,17 +171,18 @@ class Many2One(WidgetInterface):
                     if result:
                         self.field.set_client(self.record,
                             self.value_from_id(*result[0]), force_change=True)
+                    else:
+                        self.wid_text.set_text('')
                     self.focus_out = True
-                    self.display(self.record, self.field)
                     self.changed = True
 
                 WinSearch(model, callback, sel_multi=False,
                     ids=ids, context=context, domain=domain,
                     view_ids=self.attrs.get('view_ids', '').split(','),
-                    views_preload=self.attrs.get('views', {}))
+                    views_preload=self.attrs.get('views', {}),
+                    new=self.but_new.get_property('sensitive'))
                 return
         self.focus_out = True
-        self.display(self.record, self.field)
         self.changed = True
         return
 
@@ -173,19 +194,24 @@ class Many2One(WidgetInterface):
             views_preload=self.attrs.get('views', {}), readonly=self._readonly)
 
     def sig_new(self, *args):
+        model = self.get_model()
+        if not model or not common.MODELACCESS[model]['create']:
+            return
         self.focus_out = False
         screen = self.get_screen()
 
         def callback(result):
-            if result and screen.save_current():
+            if result:
                 self.field.set_client(self.record,
                     self.value_from_id(screen.current_record.id,
                         screen.current_record.rec_name()))
-                self.display(self.record, self.field)
             self.focus_out = True
-        WinForm(screen, callback, new=True)
+        WinForm(screen, callback, new=True, save_current=True)
 
     def sig_edit(self, widget):
+        model = self.get_model()
+        if not model or not common.MODELACCESS[model]['read']:
+            return
         if not self.focus_out or not self.field:
             return
         self.changed = False
@@ -198,18 +224,14 @@ class Many2One(WidgetInterface):
             screen.load([self.id_from_value(self.field.get(self.record))])
 
             def callback(result):
-                if result and screen.save_current():
+                if result:
                     self.field.set_client(self.record,
                         self.value_from_id(screen.current_record.id,
                             screen.current_record.rec_name()),
                         force_change=True)
-                elif result:
-                    screen.display()
-                    return WinForm(screen, callback)
                 self.focus_out = True
-                self.display(self.record, self.field)
                 self.changed = True
-            WinForm(screen, callback)
+            WinForm(screen, callback, save_current=True)
             return
         elif model and not self._readonly:
             domain = self.field.domain_get(self.record)
@@ -233,7 +255,6 @@ class Many2One(WidgetInterface):
                 self.field.set_client(self.record, self.value_from_id(ids[0]),
                     force_change=True)
                 self.focus_out = True
-                self.display(self.record, self.field)
                 return True
 
             def callback(result):
@@ -241,29 +262,36 @@ class Many2One(WidgetInterface):
                     self.field.set_client(self.record,
                         self.value_from_id(*result[0]), force_change=True)
                 self.focus_out = True
-                self.display(self.record, self.field)
                 self.changed = True
             WinSearch(model, callback, sel_multi=False,
                 ids=ids, context=context, domain=domain,
                 view_ids=self.attrs.get('view_ids', '').split(','),
-                views_preload=self.attrs.get('views', {}))
+                views_preload=self.attrs.get('views', {}),
+                new=self.but_new.get_property('sensitive'))
             return
         self.focus_out = True
-        self.display(self.record, self.field)
         self.changed = True
         return
 
     def sig_key_press(self, widget, event, *args):
         editable = self.wid_text.get_editable()
-        if event.keyval == gtk.keysyms.F3 and editable:
+        if (event.keyval == gtk.keysyms.F3
+                and editable
+                and self.but_new.get_property('sensitive')):
             self.sig_new(widget, event)
             return True
-        elif event.keyval == gtk.keysyms.F2:
+        elif (event.keyval == gtk.keysyms.F2
+                and self.but_open.get_property('sensitive')):
             self.sig_edit(widget)
             return True
         elif (event.keyval in (gtk.keysyms.Tab, gtk.keysyms.Return)
                 and editable):
             self.sig_activate(widget, event, key_press=True)
+        elif (self.has_target(self.field.get(self.record))
+                and editable
+                and event.keyval in (gtk.keysyms.Delete,
+                    gtk.keysyms.BackSpace)):
+            self.wid_text.set_text('')
         return False
 
     def sig_changed(self, *args):
@@ -272,14 +300,19 @@ class Many2One(WidgetInterface):
         value = self.field.get(self.record)
         if self.has_target(value):
             def clean():
+                text = self.wid_text.get_text()
+                position = self.wid_text.get_position()
                 self.field.set_client(self.record,
                     self.value_from_id(None, ''))
-                self.display(self.record, self.field)
+                # Restore text and position after display
+                self.wid_text.set_text(text)
+                self.wid_text.set_position(position)
             gobject.idle_add(clean)
         return False
 
     def set_value(self, record, field):
-        pass  # No update of the model, the model is updated in real time !
+        # Simulate a focus-out
+        self.sig_activate()
 
     def set_text(self, value):
         if not value:
@@ -290,6 +323,9 @@ class Many2One(WidgetInterface):
     def display(self, record, field):
         self.changed = False
         super(Many2One, self).display(record, field)
+
+        self._set_button_sensitive()
+
         if not field:
             self.wid_text.set_text('')
             self.wid_text.set_position(0)
@@ -310,5 +346,7 @@ class Many2One(WidgetInterface):
     def _populate_popup(self, widget, menu):
         value = self.field.get(self.record)
         if self.has_target(value):
-            populate(menu, self.get_model(), self.id_from_value(value))
+            # Delay filling of popup as it can take time
+            gobject.idle_add(populate, menu, self.get_model(),
+                self.id_from_value(value))
         return True
