@@ -11,11 +11,11 @@ from tryton.common.button import Button
 _ = gettext.gettext
 
 
-class Label(gtk.Label):
+class StateMixin(object):
 
-    def __init__(self, str=None, attrs=None):
-        super(Label, self).__init__(str=str)
-        self.attrs = attrs or {}
+    def __init__(self, *args, **kwargs):
+        self.attrs = kwargs.pop('attrs')
+        super(StateMixin, self).__init__(*args, **kwargs)
 
     def state_set(self, record):
         if record:
@@ -29,64 +29,48 @@ class Label(gtk.Label):
             self.show()
 
 
-class VBox(gtk.VBox):
-
-    def __init__(self, homogeneous=False, spacing=0, attrs=None):
-        super(VBox, self).__init__(homogeneous, spacing)
-        self.attrs = attrs or {}
-
-    def state_set(self, record):
-        if record:
-            state_changes = record.expr_eval(self.attrs.get('states', {}),
-                    check_load=False)
-        else:
-            state_changes = {}
-        if state_changes.get('invisible', self.attrs.get('invisible')):
-            self.hide()
-        else:
-            self.show()
+class Label(StateMixin, gtk.Label):
+    pass
 
 
-class Image(gtk.Image):
-
-    def __init__(self, attrs=None):
-        super(Image, self).__init__()
-        self.attrs = attrs or {}
-
-    def state_set(self, record):
-        if record:
-            state_changes = record.expr_eval(self.attrs.get('states', {}),
-                    check_load=False)
-        else:
-            state_changes = {}
-        if state_changes.get('invisible', self.attrs.get('invisible')):
-            self.hide()
-        else:
-            self.show()
-        state_changes = self.attrs.get('states', {})
+class VBox(StateMixin, gtk.VBox):
+    pass
 
 
-class Frame(gtk.Frame):
+class Image(StateMixin, gtk.Image):
+    pass
 
-    def __init__(self, label=None, attrs=None):
+
+class Frame(StateMixin, gtk.Frame):
+
+    def __init__(self, label=None, attrs=None, widgets=None):
         if not label:  # label must be None to have no label widget
             label = None
-        super(Frame, self).__init__(label=label)
-        self.attrs = attrs or {}
+        super(Frame, self).__init__(label=label, attrs=attrs)
+        self.widgets = widgets or {}
         if not label:
             self.set_shadow_type(gtk.SHADOW_NONE)
         self.set_border_width(0)
 
+
+class ScrolledWindow(StateMixin, gtk.ScrolledWindow):
+    pass
+
+
+class Notebook(StateMixin, gtk.Notebook):
+
     def state_set(self, record):
+        super(Notebook, self).state_set(record)
         if record:
             state_changes = record.expr_eval(self.attrs.get('states', {}),
                     check_load=False)
         else:
             state_changes = {}
-        if state_changes.get('invisible', self.attrs.get('invisible')):
-            self.hide()
-        else:
-            self.show()
+        if state_changes.get('readonly', self.attrs.get('readonly')):
+            for widgets in self.widgets.itervalues():
+                for widget in widgets:
+                    widget._readonly_set(True)
+                    widget.color_set('readonly')
 
 
 class ScrolledWindow(gtk.ScrolledWindow):
@@ -114,8 +98,8 @@ class Alignment(gtk.Alignment):
         super(Alignment, self).__init__(
             float(attrs.get('xalign', 0.0)),
             float(attrs.get('yalign', 0.5)),
-            abs(1 - float(attrs.get('xalign', 0.0))),
-            abs(1 - float(attrs.get('yalign', 0.0))))
+            float(attrs.get('xexpand', 1.0)),
+            float(attrs.get('yexpand', 1.0)))
         self.add(widget)
         widget.connect('show', lambda *a: self.show())
         widget.connect('hide', lambda *a: self.hide())
@@ -203,7 +187,7 @@ class ParserForm(ParserInterface):
     def parse(self, model_name, root_node, fields, notebook=None, paned=None,
             tooltips=None):
         dict_widget = {}
-        button_list = []
+        state_widgets = []
         notebook_list = []
         attrs = common.node_attributes(root_node)
         on_write = attrs.get('on_write', '')
@@ -234,8 +218,8 @@ class ParserForm(ParserInterface):
             colspan = int(attrs.get('colspan', 1))
             if node.localName == 'image':
                 common.ICONFACTORY.register_icon(attrs['name'])
-                icon = Image(attrs)
-                button_list.append(icon)
+                icon = Image(attrs=attrs)
+                state_widgets.append(icon)
                 icon.set_from_stock(attrs['name'], gtk.ICON_SIZE_DIALOG)
                 container.wid_add(icon,
                     help_tip=attrs.get('help', False),
@@ -253,10 +237,9 @@ class ParserForm(ParserInterface):
                     if not text:
                         text = fields[attrs['name']].attrs['string']
                 vbox = VBox(attrs=attrs)
-                button_list.append(vbox)
+                state_widgets.append(vbox)
                 if text:
                     label = gtk.Label(text)
-                    label.set_use_markup(True)
                     label.set_alignment(float(attrs.get('xalign', 0.0)), 0.5)
                     vbox.pack_start(label)
                 vbox.pack_start(gtk.HSeparator())
@@ -295,9 +278,8 @@ class ParserForm(ParserInterface):
                 if not text:
                     container.empty_add(int(attrs.get('colspan', 1)))
                     continue
-                label = Label(text, attrs)
-                button_list.append(label)
-                label.set_use_markup(True)
+                label = Label(text, attrs=attrs)
+                state_widgets.append(label)
                 if CONFIG['client.modepda']:
                     attrs['xalign'] = 0.0
                 label.set_alignment(float(attrs.get('xalign', 1.0)),
@@ -315,7 +297,7 @@ class ParserForm(ParserInterface):
 
             elif node.localName == 'button':
                 button = Button(attrs)
-                button_list.append(button)
+                state_widgets.append(button)
                 container.wid_add(button,
                     help_tip=attrs.get('help', False),
                     colspan=colspan,
@@ -323,7 +305,8 @@ class ParserForm(ParserInterface):
                     xexpand=xexpand, xfill=xfill)
 
             elif node.localName == 'notebook':
-                notebook = gtk.Notebook()
+                notebook = Notebook(attrs=attrs)
+                state_widgets.append(notebook)
                 notebook.set_scrollable(True)
                 notebook_list.append(notebook)
                 if CONFIG['client.form_tab'] == 'top':
@@ -343,13 +326,13 @@ class ParserForm(ParserInterface):
                     colspan=colspan,
                     yexpand=yexpand, yfill=yfill,
                     xexpand=xexpand, xfill=xfill)
-                widget, widgets, buttons, spam, notebook_list2, cursor_widget2\
-                    = self.parse(model_name, node, fields, notebook,
-                        tooltips=tooltips)
+                (widget, widgets, state_widgets2, spam, notebook_list2,
+                    cursor_widget2) = self.parse(model_name, node, fields,
+                        notebook, tooltips=tooltips)
                 if not cursor_widget:
                     cursor_widget = cursor_widget2
                 notebook_list.extend(notebook_list2)
-                button_list += buttons
+                state_widgets += state_widgets2
                 for widget_name, widgets in widgets.items():
                     dict_widget.setdefault(widget_name, [])
                     dict_widget[widget_name].extend(widgets)
@@ -400,13 +383,13 @@ class ParserForm(ParserInterface):
                     else:
                         tab_box.pack_start(icon)
                 tab_box.show_all()
-                widget, widgets, buttons, spam, notebook_list2, cursor_widget2\
-                    = self.parse(model_name, node, fields, notebook,
-                        tooltips=tooltips)
+                (widget, widgets, state_widgets2, spam, notebook_list2,
+                    cursor_widget2) = self.parse(model_name, node, fields,
+                        notebook, tooltips=tooltips)
                 if not cursor_widget:
                     cursor_widget = cursor_widget2
                 notebook_list.extend(notebook_list2)
-                button_list += buttons
+                state_widgets += state_widgets2
                 for widget_name, widgets in widgets.iteritems():
                     dict_widget.setdefault(widget_name, [])
                     dict_widget[widget_name].extend(widgets)
@@ -421,7 +404,7 @@ class ParserForm(ParserInterface):
                         gtk.POLICY_AUTOMATIC)
                 scrolledwindow.add(viewport)
 
-                button_list.append(scrolledwindow)
+                state_widgets.append(scrolledwindow)
                 notebook.append_page(scrolledwindow, tab_box)
 
             elif node.localName == 'field':
@@ -431,7 +414,7 @@ class ParserForm(ParserInterface):
                     continue
                 if name not in fields:
                     container.empty_add(int(attrs.get('colspan', 1)))
-                    log = logging.getLogger('view')
+                    log = logging.getLogger(__name__)
                     log.error('Unknown field "%s"' % str(name))
                     continue
                 ftype = attrs.get('widget', fields[name].attrs['type'])
@@ -441,7 +424,7 @@ class ParserForm(ParserInterface):
                 for attr_name in ('relation', 'domain', 'selection',
                         'relation_field', 'string', 'views', 'invisible',
                         'add_remove', 'sort', 'context', 'size', 'filename',
-                        'autocomplete', 'translate'):
+                        'autocomplete', 'translate', 'create', 'delete'):
                     if attr_name in fields[name].attrs and \
                             not attr_name in attrs:
                         attrs[attr_name] = fields[name].attrs[attr_name]
@@ -466,15 +449,16 @@ class ParserForm(ParserInterface):
                     xexpand=xexpand, xfill=xfill)
 
             elif node.localName == 'group':
-                widget, widgets, buttons, spam, notebook_list2, cursor_widget2\
-                    = self.parse(model_name, node, fields, tooltips=tooltips)
+                (widget, widgets, state_widgets2, spam, notebook_list2,
+                    cursor_widget2) = self.parse(model_name, node, fields,
+                        tooltips=tooltips)
                 if not cursor_widget:
                     cursor_widget = cursor_widget2
                 notebook_list.extend(notebook_list2)
-                for widget_name, widgets in widgets.iteritems():
+                for widget_name, lwidgets in widgets.iteritems():
                     dict_widget.setdefault(widget_name, [])
-                    dict_widget[widget_name].extend(widgets)
-                button_list += buttons
+                    dict_widget[widget_name].extend(lwidgets)
+                state_widgets += state_widgets2
                 text = ''
                 if 'name' in attrs and attrs['name'] in fields:
                     if attrs['name'] == self.screen.exclude_field:
@@ -488,9 +472,9 @@ class ParserForm(ParserInterface):
                 if attrs.get('string'):
                     text = attrs['string']
 
-                frame = Frame(text, attrs)
+                frame = Frame(text, attrs=attrs, widgets=widgets)
                 frame.add(widget)
-                button_list.append(frame)
+                state_widgets.append(frame)
                 container.wid_add(frame,
                     colspan=colspan,
                     yexpand=yexpand, yfill=yfill, ypadding=0,
@@ -499,13 +483,13 @@ class ParserForm(ParserInterface):
                 hpaned = gtk.HPaned()
                 container.wid_add(hpaned, colspan=int(attrs.get('colspan', 4)),
                         yexpand=True, yfill=True)
-                widget, widgets, buttons, spam, notebook_list2, cursor_widget2\
-                    = self.parse(model_name, node, fields, paned=hpaned,
-                        tooltips=tooltips)
+                (widget, widgets, state_widgets, spam, notebook_list2,
+                    cursor_widget2) = self.parse(model_name, node, fields,
+                        paned=hpaned, tooltips=tooltips)
                 if not cursor_widget:
                     cursor_widget = cursor_widget2
                 notebook_list.extend(notebook_list2)
-                button_list += buttons
+                state_widgets += state_widgets
                 for widget_name, widgets in widgets.iteritems():
                     dict_widget.setdefault(widget_name, [])
                     dict_widget[widget_name].extend(widgets)
@@ -515,26 +499,26 @@ class ParserForm(ParserInterface):
                 vpaned = gtk.VPaned()
                 container.wid_add(vpaned, colspan=int(attrs.get('colspan', 4)),
                         yexpand=True, yfill=True)
-                widget, widgets, buttons, spam, notebook_list, cursor_widget2\
-                    = self.parse(model_name, node, fields, paned=vpaned,
-                        tooltips=tooltips)
+                (widget, widgets, state_widgets2, spam, notebook_list,
+                    cursor_widget2) = self.parse(model_name, node, fields,
+                        paned=vpaned, tooltips=tooltips)
                 if not cursor_widget:
                     cursor_widget = cursor_widget2
                 notebook_list.extend(notebook_list2)
-                button_list += buttons
+                state_widgets += state_widgets2
                 for widget_name, widgets in widgets.iteritems():
                     dict_widget.setdefault(widget_name, [])
                     dict_widget[widget_name].extend(widgets)
                 if 'position' in attrs:
                     vpaned.set_position(int(attrs['position']))
             elif node.localName == 'child':
-                widget, widgets, buttons, spam, notebook_list2, cursor_widget2\
-                    = self.parse(model_name, node, fields, paned=paned,
-                        tooltips=tooltips)
+                (widget, widgets, state_widgets2, spam, notebook_list2,
+                    cursor_widget2) = self.parse(model_name, node, fields,
+                        paned=paned, tooltips=tooltips)
                 if not cursor_widget:
                     cursor_widget = cursor_widget2
                 notebook_list.extend(notebook_list2)
-                button_list += buttons
+                state_widgets += state_widgets2
                 for widget_name, widgets in widgets.iteritems():
                     dict_widget.setdefault(widget_name, [])
                     dict_widget[widget_name].extend(widgets)
@@ -542,7 +526,7 @@ class ParserForm(ParserInterface):
                     paned.pack1(widget, resize=True, shrink=True)
                 elif not paned.get_child2():
                     paned.pack2(widget, resize=True, shrink=True)
-        return container.pop(), dict_widget, button_list, on_write, \
+        return container.pop(), dict_widget, state_widgets, on_write, \
                 notebook_list, cursor_widget
 
 from calendar import Calendar, DateTime, Time
