@@ -15,7 +15,6 @@ import gettext
 from tryton.config import CONFIG
 from tryton.common.cellrendererbutton import CellRendererButton
 from tryton.common.cellrenderertoggle import CellRendererToggle
-from tryton.common.cellrendererbinary import CellRendererBinary
 from tryton.pyson import PYSONEncoder
 from tryton.gui.window import Window
 from tryton.common.popup_menu import populate
@@ -74,14 +73,6 @@ class AdaptModelGroup(gtk.GenericTreeModel):
                     or record.group.child_name == self.children_field)):
             path = self.on_get_path(record)
             self.row_deleted(path)
-            if (record.parent and
-                    record.group != self.group and
-                    len(record.children_group(self.children_field)) <= 1):
-                path = self.on_get_path(record.parent)
-                iter_ = self.get_iter(path)
-                self.__removed = record  # XXX check for thread
-                self.row_has_child_toggled(path, iter_)
-                self.__removed = None
 
     def append(self, model):
         self.group.add(model)
@@ -126,6 +117,10 @@ class AdaptModelGroup(gtk.GenericTreeModel):
         group = parent.children_group(self.children_field, check_load=True)
         if group is not record.group:
             record.group.remove(record, remove=True, force_remove=True)
+            # Don't remove record from previous group
+            # as the new parent will change the parent
+            # This prevents concurrency conflict
+            record.group.record_removed.remove(record)
             group.add(record)
             record.modified_fields.setdefault(record.parent_name or 'id')
         group.move(record, 0)
@@ -251,9 +246,9 @@ class AdaptModelGroup(gtk.GenericTreeModel):
 
 class ViewList(ParserView):
 
-    def __init__(self, screen, widget, children=None, buttons=None,
+    def __init__(self, screen, widget, children=None, state_widgets=None,
             notebooks=None, cursor_widget=None, children_field=None):
-        super(ViewList, self).__init__(screen, widget, children, buttons,
+        super(ViewList, self).__init__(screen, widget, children, state_widgets,
             notebooks, cursor_widget, children_field)
         self.store = None
         self.view_type = 'tree'
@@ -346,6 +341,10 @@ class ViewList(ParserView):
                 and event.state & gtk.gdk.CONTROL_MASK):
             self.on_copy()
             return False
+        if (event.keyval == gtk.keysyms.v
+                and event.state & gtk.gdk.CONTROL_MASK):
+            self.on_paste()
+            return False
         if event.keyval in (gtk.keysyms.Down, gtk.keysyms.Up):
             path, column = widget.get_cursor()
             if not path:
@@ -447,6 +446,53 @@ class ViewList(ParserView):
         del selection
         return
 
+    def on_paste(self):
+        if (not hasattr(self.widget_tree, 'editable')
+                or not self.widget_tree.editable):
+            return
+
+        def unquote(value):
+            if value[:1] == '"' and value[-1:] == '"':
+                return value[1:-1]
+            return value
+        data = []
+        for clipboard_type in (gtk.gdk.SELECTION_CLIPBOARD,
+                gtk.gdk.SELECTION_PRIMARY):
+            clipboard = self.widget_tree.get_clipboard(clipboard_type)
+            text = clipboard.wait_for_text()
+            if not text:
+                continue
+            data = [[unquote(v) for v in l.split('\t')]
+                for l in text.splitlines()]
+            break
+        col = self.widget_tree.get_cursor()[1]
+        columns = [c for c in self.widget_tree.get_columns()
+            if c.get_visible() and c.name]
+        if col in columns:
+            idx = columns.index(col)
+            columns = columns[idx:]
+        record = self.screen.current_record
+        group = record.group
+        idx = group.index(record)
+        for line in data:
+            record = group[idx]
+            for col, value in zip(columns, line):
+                cell = self.widget_tree.cells[col.name]
+                if cell.get_textual_value(record) != value:
+                    cell.value_from_text(record, value)
+                    if cell.get_textual_value(record) != value:
+                        # Stop setting value if a value is correctly set
+                        idx = len(group)
+                        break
+            if not record.validate():
+                break
+            idx += 1
+            if idx >= len(group):
+                # TODO create new record
+                break
+        self.screen.current_record = record
+        self.screen.display(set_cursor=True)
+
     def drag_begin(self, treeview, context):
         return True
 
@@ -549,38 +595,44 @@ class ViewList(ParserView):
                 return False
             group = model.group
             record = group[path[0][0]]
-
             menu = gtk.Menu()
-            copy_item = gtk.ImageMenuItem('gtk-copy')
-            copy_item.connect('activate', lambda x: self.on_copy())
-            menu.append(copy_item)
-            # Don't activate actions if parent is modified
-            parent = record.parent if record else None
-            while parent:
-                if parent.modified:
-                    record = None
-                    break
-                parent = parent.parent
-            populate(menu, group.model_name, record)
-            for col in self.widget_tree.get_columns():
-                if not col.get_visible() or not col.name:
-                    continue
-                field = group.fields[col.name]
-                model = None
-                if field.attrs['type'] == 'many2one':
-                    model = field.attrs['relation']
-                    record_id = field.get(record)
-                elif field.attrs['type'] == 'reference':
-                    value = field.get(record)
-                    if value:
-                        model, record_id = value.split(',')
-                        record_id = int(record_id)
-                if not model:
-                    continue
-                label = field.attrs['string']
-                populate(menu, model, record_id, title=label)
-            menu.show_all()
             menu.popup(None, None, None, event.button, event.time)
+
+            def pop(menu, group, record):
+                copy_item = gtk.ImageMenuItem('gtk-copy')
+                copy_item.connect('activate', lambda x: self.on_copy())
+                menu.append(copy_item)
+                paste_item = gtk.ImageMenuItem('gtk-paste')
+                paste_item.connect('activate', lambda x: self.on_paste())
+                menu.append(paste_item)
+                # Don't activate actions if parent is modified
+                parent = record.parent if record else None
+                while parent:
+                    if parent.modified:
+                        record = None
+                        break
+                    parent = parent.parent
+                populate(menu, group.model_name, record)
+                for col in self.widget_tree.get_columns():
+                    if not col.get_visible() or not col.name:
+                        continue
+                    field = group.fields[col.name]
+                    model = None
+                    if field.attrs['type'] == 'many2one':
+                        model = field.attrs['relation']
+                        record_id = field.get(record)
+                    elif field.attrs['type'] == 'reference':
+                        value = field.get(record)
+                        if value:
+                            model, record_id = value.split(',')
+                            record_id = int(record_id)
+                    if not model:
+                        continue
+                    label = field.attrs['string']
+                    populate(menu, model, record_id, title=label)
+                menu.show_all()
+            # Delay filling of popup as it can take time
+            gobject.idle_add(pop, menu, group, record)
         elif event.button == 2:
             event.button = 1
             event.state |= gtk.gdk.MOD1_MASK
@@ -653,11 +705,13 @@ class ViewList(ParserView):
             del fields[last_col.name]
 
         if fields and any(fields.itervalues()):
+            model_name = self.screen.model_name
             try:
                 RPCExecute('model', 'ir.ui.view_tree_width', 'set_width',
-                    self.screen.model_name, fields)
+                    model_name, fields)
             except RPCException:
                 pass
+            self.screen.tree_column_width[model_name].update(fields)
 
     def destroy(self):
         self.save_width_height()
@@ -703,16 +757,34 @@ class ViewList(ParserView):
             else:
                 self.screen.current_record = None
 
-        if hasattr(self.widget_tree, 'editable') \
-                and self.widget_tree.editable \
-                and not self.screen.parent \
-                and previous_record != self.screen.current_record:
-            if previous_record and \
-                    not (previous_record.validate(self.get_fields())
-                            and previous_record.save()):
+        if (hasattr(self.widget_tree, 'editable')
+                and self.widget_tree.editable
+                and previous_record):
+            def go_previous():
                 self.screen.current_record = previous_record
                 self.set_cursor()
-                return True
+            if (not self.screen.parent
+                    and previous_record != self.screen.current_record):
+
+                def save():
+                    if not previous_record.destroyed:
+                        if not previous_record.save():
+                            go_previous()
+
+                if not previous_record.validate(self.get_fields()):
+                    go_previous()
+                    return True
+                # Delay the save to let GTK process the current event
+                gobject.idle_add(save)
+            elif (previous_record != self.screen.current_record
+                    and self.screen.pre_validate):
+
+                def pre_validate():
+                    if not previous_record.destroyed:
+                        if not previous_record.pre_validate():
+                            go_previous()
+                # Delay the pre_validate to let GTK process the current event
+                gobject.idle_add(pre_validate)
         self.update_children()
 
     def set_value(self):
@@ -788,23 +860,7 @@ class ViewList(ParserView):
             path = self.store.on_get_path(self.screen.current_record)
             if self.store.get_flags() & gtk.TREE_MODEL_LIST_ONLY:
                 path = (path[0],)
-            focus_column = None
-            for column in self.widget_tree.get_columns():
-                renderers = column.get_cell_renderers()
-                if not renderers:
-                    continue
-                renderer = renderers[0]
-                if isinstance(renderer, CellRendererToggle):
-                    editable = renderer.get_property('activatable')
-                elif isinstance(renderer,
-                        (gtk.CellRendererProgress, CellRendererButton,
-                            gtk.CellRendererPixbuf, CellRendererBinary)):
-                    editable = False
-                else:
-                    editable = renderer.get_property('editable')
-                if column.get_visible() and editable:
-                    focus_column = column
-                    break
+            focus_column = self.widget_tree.next_column(path)
             if path[:-1]:
                 self.widget_tree.expand_to_path(path[:-1])
             self.widget_tree.scroll_to_cell(path, focus_column,
