@@ -11,14 +11,14 @@ import operator
 
 from functools import wraps
 
-from editabletree import EditableTreeView
+from editabletree import EditableTreeView, TreeView
 from tryton.gui.window.view_form.view.interface import ParserInterface
 from tryton.gui.window.win_search import WinSearch
 from tryton.gui.window.win_form import WinForm
 from tryton.gui.window.view_form.screen import Screen
 import tryton.rpc as rpc
 from tryton.common import COLORS, node_attributes, \
-        HM_FORMAT, file_selection, file_open
+        file_selection, file_open
 import tryton.common as common
 from tryton.common.cellrendererbutton import CellRendererButton
 from tryton.common.cellrendererdate import CellRendererDate
@@ -28,7 +28,6 @@ from tryton.common.cellrenderercombo import CellRendererCombo
 from tryton.common.cellrendererinteger import CellRendererInteger
 from tryton.common.cellrendererfloat import CellRendererFloat
 from tryton.common.cellrendererbinary import CellRendererBinary
-from tryton.action import Action
 from tryton.translate import date_format
 from tryton.common import RPCExecute, RPCException
 
@@ -64,13 +63,11 @@ def sort_model(column, treeview, screen):
     store = treeview.get_model()
     unsaved_records = [x for x in store.group if x.id < 0]
     search_string = screen.screen_container.get_text() or None
-    if screen.search_count == len(store):
+    if screen.search_count == len(store) or unsaved_records:
         ids = screen.search_filter(search_string=search_string, only_ids=True)
         store.sort(ids)
     else:
         screen.search_filter(search_string=search_string)
-    for record in unsaved_records:
-        store.group.append(record)
 
 
 def realized(func):
@@ -94,14 +91,14 @@ class ParserTree(ParserInterface):
 
     def parse(self, model_name, root_node, fields):
         dict_widget = {}
-        button_list = []
+        state_widgets = []
         attrs = node_attributes(root_node)
         on_write = attrs.get('on_write', '')
         editable = attrs.get('editable', False)
         if editable:
             treeview = EditableTreeView(editable)
         else:
-            treeview = gtk.TreeView()
+            treeview = TreeView()
             treeview.cells = {}
         treeview.sequence = attrs.get('sequence', False)
         treeview.colors = attrs.get('colors', '"black"')
@@ -197,8 +194,8 @@ class ParserTree(ParserInterface):
                     'float': 80,
                     'numeric': 80,
                     'float_time': 100,
-                    'date': 90,
-                    'datetime': 140,
+                    'date': 110,
+                    'datetime': 160,
                     'selection': 90,
                     'char': 100,
                     'one2many': 50,
@@ -206,10 +203,12 @@ class ParserTree(ParserInterface):
                     'boolean': 20,
                     'binary': 200,
                 }
-                if 'width' in node_attrs:
-                    width = int(node_attrs['width'])
-                else:
-                    width = twidth.get(fields[fname].attrs['type'], 100)
+                width = self.screen.tree_column_width[model_name].get(fname)
+                if not width:
+                    if 'width' in node_attrs:
+                        width = int(node_attrs['width'])
+                    else:
+                        width = twidth.get(fields[fname].attrs['type'], 100)
                 col.width = width
                 if width > 0:
                     col.set_fixed_width(width)
@@ -242,7 +241,7 @@ class ParserTree(ParserInterface):
             elif node.localName == 'button':
                 #TODO add shortcut
                 cell = Button(treeview, self.screen, node_attrs)
-                button_list.append(cell)
+                state_widgets.append(cell)
                 renderer = cell.renderer
                 string = node_attrs.get('string', _('Unknown'))
                 col = gtk.TreeViewColumn(string, renderer)
@@ -282,7 +281,7 @@ class ParserTree(ParserInterface):
             col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
             treeview.append_column(col)
         treeview.set_fixed_height_mode(True)
-        return treeview, dict_widget, button_list, on_write, [], None
+        return treeview, dict_widget, state_widgets, on_write, [], None
 
     def set_selection(self, treeview, direction):
         selection = treeview.get_selection()
@@ -435,7 +434,14 @@ class Datetime(Date):
     def __init__(self, field_name, model_name, treeview, attrs=None):
         super(Datetime, self).__init__(field_name, model_name, treeview,
             attrs=attrs)
-        self.renderer.format = date_format() + ' ' + HM_FORMAT
+
+    @realized
+    def setter(self, column, cell, store, iter):
+        super(Datetime, self).setter(column, cell, store, iter)
+        record = store.get_value(iter, 0)
+        field = record[self.field_name]
+        time_format = field.time_format(record)
+        self.renderer.format = date_format() + ' ' + time_format
 
 
 class Time(Date):
@@ -443,7 +449,14 @@ class Time(Date):
     def __init__(self, field_name, model_name, treeview, attrs=None):
         super(Time, self).__init__(field_name, model_name, treeview,
             attrs=attrs)
-        self.renderer.format = HM_FORMAT
+
+    @realized
+    def setter(self, column, cell, store, iter):
+        super(Time, self).setter(column, cell, store, iter)
+        record = store.get_value(iter, 0)
+        field = record[self.field_name]
+        time_format = field.time_format(record)
+        self.renderer.format = time_format
 
 
 class Float(Char):
@@ -583,9 +596,8 @@ class M2O(Char):
                 callback()
             return
         if len(ids) != 1:
-            if callback:
-                self.search_remote(record, relation, ids, domain=domain,
-                    context=context, callback=callback)
+            self.search_remote(record, relation, ids, domain=domain,
+                context=context, callback=callback)
             return
         field.set_client(record, ids[0])
         if callback:
@@ -596,6 +608,12 @@ class M2O(Char):
         field = record.group.fields[self.field_name]
         relation = field.attrs['relation']
 
+        access = common.MODELACCESS[relation]
+        if create and not access['create']:
+            return
+        elif not access['read']:
+            return
+
         domain = field.domain_get(record)
         context = field.context_get(record)
         if create:
@@ -627,20 +645,17 @@ class M2O(Char):
             mode=['form'])
 
         def open_callback(result):
-            if result and screen.save_current():
+            if result:
                 value = (screen.current_record.id,
                     screen.current_record.rec_name())
                 field.set_client(record, value, force_change=True)
-            elif result:
-                screen.display()
-                return WinForm(screen, open_callback)
             if callback:
                 callback()
         if obj_id:
             screen.load([obj_id])
-            WinForm(screen, open_callback)
+            WinForm(screen, open_callback, save_current=True)
         else:
-            WinForm(screen, open_callback, new=True)
+            WinForm(screen, open_callback, new=True, save_current=True)
 
     def search_remote(self, record, relation, ids=None, domain=None,
             context=None, callback=None):
@@ -689,8 +704,13 @@ class O2M(Char):
         relation = field.attrs['relation']
         context = field.context_get(record)
 
+        access = common.MODELACCESS[relation]
+        if not access['read']:
+            return
+
         screen = Screen(relation, mode=['tree', 'form'],
             exclude_field=field.attrs.get('relation_field'))
+        screen.pre_validate = bool(int(self.attrs.get('pre_validate', 0)))
         screen.group = group
 
         def open_callback(result):
@@ -924,38 +944,7 @@ class Button(object):
         if state_changes.get('invisible') \
                 or state_changes.get('readonly'):
             return True
-
-        self.screen.current_record = record
-        obj_id = self.screen.save_current()
-        if obj_id:
-            if not self.attrs.get('confirm', False) or \
-                    common.sur(self.attrs['confirm']):
-                button_type = self.attrs.get('type', 'object')
-                ctx = record.context_get()
-                if button_type == 'object':
-                    try:
-                        RPCExecute('model', self.screen.model_name,
-                            self.attrs['name'], [obj_id], context=ctx)
-                    except RPCException:
-                        pass
-                elif button_type == 'action':
-                    try:
-                        action_id = RPCExecute('model', 'ir.action',
-                            'get_action_id', int(self.attrs['name']),
-                            context=ctx)
-                    except RPCException:
-                        action_id = None
-                    if action_id:
-                        Action.execute(action_id, {
-                            'model': self.screen.model_name,
-                            'id': obj_id,
-                            'ids': [obj_id],
-                            }, context=ctx)
-                else:
-                    raise Exception('Unallowed button type')
-                self.screen.reload(written=True)
-            else:
-                self.screen.display()
+        self.screen.button(self.attrs)
 
 CELLTYPES = {
     'char': Char,
