@@ -16,12 +16,15 @@ class WinForm(NoModal):
     "Form window"
 
     def __init__(self, screen, callback, view_type='form',
-            new=False, many=False, domain=None, context=None):
+            new=False, many=0, domain=None, context=None,
+            save_current=False):
         NoModal.__init__(self)
         self.screen = screen
         self.callback = callback
+        self.many = many
         self.domain = domain
         self.context = context
+        self.save_current = save_current
         self.prev_view = self.screen.current_view
         self.screen.screen_container.alternate_view = True
         switch_new = False
@@ -39,6 +42,9 @@ class WinForm(NoModal):
         self.win.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.win.set_icon(TRYTON_ICON)
         self.win.set_has_separator(False)
+        if hasattr(self.win, 'set_deletable'):
+            self.win.set_deletable(False)
+        self.win.connect('close', self.close)
         self.win.connect('response', self.response)
 
         self.accel_group = gtk.AccelGroup()
@@ -47,37 +53,40 @@ class WinForm(NoModal):
         self.but_cancel = None
         self.but_ok = None
         self.but_new = None
-        if new:
+        if new or self.save_current:
             icon_cancel = gtk.STOCK_CANCEL
             self.but_cancel = self.win.add_button(icon_cancel,
                     gtk.RESPONSE_CANCEL)
+            self.but_cancel.set_focus_on_click(False)
 
-        self.but_ok = self.win.add_button(gtk.STOCK_OK,
-            gtk.RESPONSE_OK)
-        if new and many:
-            self.but_ok.add_accelerator('clicked',
-                self.accel_group, gtk.keysyms.Return,
-                gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK,
-                gtk.ACCEL_VISIBLE)
-
+        if new and self.many:
             self.but_new = self.win.add_button(gtk.STOCK_NEW,
                 gtk.RESPONSE_ACCEPT)
-            self.but_new.add_accelerator('clicked', self.accel_group,
-                gtk.keysyms.Return, gtk.gdk.CONTROL_MASK,
-                gtk.ACCEL_VISIBLE)
-            self.win.set_default_response(gtk.RESPONSE_ACCEPT)
+            self.but_new.set_focus_on_click(False)
+            self.but_new.set_accel_path('<tryton>/Form/New', self.accel_group)
+
+        if self.save_current:
+            self.but_ok = gtk.Button(_('_Save'))
+            img_save = gtk.Image()
+            img_save.set_from_stock('tryton-save', gtk.ICON_SIZE_BUTTON)
+            self.but_ok.set_image(img_save)
+            self.but_ok.set_accel_path('<tryton>/Form/Save', self.accel_group)
+            self.but_ok.set_can_default(True)
+            self.but_ok.show()
+            self.win.add_action_widget(self.but_ok, gtk.RESPONSE_OK)
         else:
-            self.but_ok.add_accelerator('clicked', self.accel_group,
-                gtk.keysyms.Return, gtk.gdk.CONTROL_MASK,
-                gtk.ACCEL_VISIBLE)
-            self.win.set_default_response(gtk.RESPONSE_OK)
+            self.but_ok = self.win.add_button(gtk.STOCK_OK,
+                gtk.RESPONSE_OK)
+        self.but_ok.set_focus_on_click(False)
+        self.but_ok.add_accelerator('clicked', self.accel_group,
+            gtk.keysyms.Return, gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
+        self.win.set_default_response(gtk.RESPONSE_OK)
 
         self.win.set_title(self.screen.current_view.title)
 
         title = gtk.Label()
-        title.set_use_markup(True)
-        title.modify_font(pango.FontDescription("12"))
-        title.set_label('<b>' + self.screen.current_view.title + '</b>')
+        title.modify_font(pango.FontDescription("bold 12"))
+        title.set_label(self.screen.current_view.title)
         title.set_padding(20, 3)
         title.set_alignment(0.0, 0.5)
         title.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#000000"))
@@ -116,6 +125,7 @@ class WinForm(NoModal):
         if view_type == 'tree':
             hbox = gtk.HBox(homogeneous=False, spacing=0)
             tooltips = common.Tooltips()
+            access = common.MODELACCESS[screen.model_name]
 
             if domain is not None:
                 self.wid_text = gtk.Entry()
@@ -134,6 +144,8 @@ class WinForm(NoModal):
                 self.but_add.add(img_add)
                 self.but_add.set_relief(gtk.RELIEF_NONE)
                 hbox.pack_start(self.but_add, expand=False, fill=False)
+                if not access['read']:
+                    self.but_add.set_sensitive(False)
 
                 self.but_remove = gtk.Button()
                 tooltips.set_tip(self.but_remove, _('Remove <Del>'))
@@ -145,6 +157,8 @@ class WinForm(NoModal):
                 self.but_remove.add(img_remove)
                 self.but_remove.set_relief(gtk.RELIEF_NONE)
                 hbox.pack_start(self.but_remove, expand=False, fill=False)
+                if not access['read']:
+                    self.but_remove.set_sensitive(False)
 
                 hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
 
@@ -157,6 +171,8 @@ class WinForm(NoModal):
             self.but_new.add(img_new)
             self.but_new.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(self.but_new, expand=False, fill=False)
+            if not access['create']:
+                self.but_new.set_sensitive(False)
 
             self.but_del = gtk.Button()
             tooltips.set_tip(self.but_del, _('Delete selected record <Del>'))
@@ -168,6 +184,8 @@ class WinForm(NoModal):
             self.but_del.add(img_del)
             self.but_del.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(self.but_del, expand=False, fill=False)
+            if not access['delete']:
+                self.but_del.set_sensitive(False)
 
             self.but_undel = gtk.Button()
             tooltips.set_tip(self.but_undel,
@@ -221,6 +239,8 @@ class WinForm(NoModal):
             but_switch.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(but_switch, expand=False, fill=False)
 
+            but_switch.props.sensitive = screen.number_of_views > 1
+
             tooltips.enable()
 
             alignment = gtk.Alignment(1.0)
@@ -247,6 +267,12 @@ class WinForm(NoModal):
             self.screen.screen_container.alternate_viewport.connect(
                     'key-press-event', self.on_keypress)
 
+        if self.save_current:
+            self.screen.signal_connect(self, 'record-message',
+                self.activate_save)
+            self.screen.signal_connect(self, 'record-modified',
+                self.activate_save)
+
         self.register()
         self.win.show()
 
@@ -358,14 +384,40 @@ class WinForm(NoModal):
         line = '(%s/%s)' % (name, signal_data[1])
         self.label.set_text(line)
 
+    def activate_save(self, *args):
+        modified = self.screen.modified()
+        self.but_ok.props.sensitive = modified
+        self.win.set_default_response(
+            gtk.RESPONSE_OK if modified else gtk.RESPONSE_CANCEL)
+
+    def close(self, widget):
+        widget.emit_stop_by_name('close')
+        return True
+
     def response(self, win, response_id):
         validate = False
         cancel_responses = (gtk.RESPONSE_CANCEL, gtk.RESPONSE_DELETE_EVENT)
         self.screen.current_view.set_value()
+        if not self.win.get_focus():
+            return
         if (response_id not in cancel_responses
                 and self.screen.current_record is not None):
             validate = self.screen.current_record.validate(
                 self.screen.current_view.get_fields())
+            if validate and self.screen.pre_validate:
+                validate = self.screen.current_record.pre_validate()
+            if validate and self.save_current:
+                if not self.screen.save_current():
+                    validate = False
+            elif validate and self.screen.current_view.view_type == 'form':
+                view = self.screen.current_view
+                for widgets in view.widgets.itervalues():
+                    for widget in widgets:
+                        if (hasattr(widget, 'screen')
+                                and widget.screen.pre_validate):
+                            record = widget.screen.current_record
+                            if record:
+                                validate = record.pre_validate()
             if not validate:
                 self.screen.set_cursor()
                 self.screen.display()
@@ -387,6 +439,10 @@ class WinForm(NoModal):
         self.screen.new(context=self.context)
         self.screen.current_view.display()
         self.screen.set_cursor(new=True)
+        self.many -= 1
+        if self.many == 0:
+            self.but_new.set_sensitive(False)
+            self.win.set_default_response(gtk.RESPONSE_OK)
 
     def destroy(self):
         self.screen.screen_container.alternate_view = False
