@@ -13,7 +13,7 @@ from tryton.gui import Main
 from tryton.exceptions import TrytonServerError, TrytonServerUnavailable
 from tryton.gui.window.nomodal import NoModal
 from tryton.common.button import Button
-from tryton.common import RPCExecute, RPCException
+from tryton.common import RPCExecute, RPCException, TRYTON_ICON
 _ = gettext.gettext
 
 
@@ -82,8 +82,9 @@ class Wizard(object):
                     result = RPCExecute('wizard', self.action, 'execute',
                         self.session_id, data, self.state, context=ctx)
                 except RPCException, rpc_exception:
-                    if not isinstance(rpc_exception.exception,
-                            TrytonServerError):
+                    if (not isinstance(rpc_exception.exception,
+                            TrytonServerError)
+                            or not self.screen):
                         self.state = self.end_state
                     break
 
@@ -171,9 +172,8 @@ class Wizard(object):
             self._record_modified)
 
         title = gtk.Label()
-        title.set_use_markup(True)
-        title.modify_font(pango.FontDescription("14"))
-        title.set_label('<b>' + self.screen.current_view.title + '</b>')
+        title.modify_font(pango.FontDescription("bold 14"))
+        title.set_label(self.screen.current_view.title)
         title.set_padding(20, 4)
         title.set_alignment(0.0, 0.5)
         title.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#000000"))
@@ -293,6 +293,7 @@ class WizardDialog(Wizard, NoModal):
         self.dia = gtk.Dialog(self.name, self.parent,
             gtk.DIALOG_DESTROY_WITH_PARENT)
         self.dia.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
+        self.dia.set_icon(TRYTON_ICON)
         if hasattr(self.dia, 'set_deletable'):
             self.dia.set_deletable(False)
         self.dia.connect('close', self.close)
@@ -346,8 +347,11 @@ class WizardDialog(Wizard, NoModal):
             dialog = self.page.dialogs[-1]
         else:
             dialog = self.page
-        if hasattr(dialog, 'screen'):
-            dialog.screen.reload(written=True)
+        if (hasattr(dialog, 'screen')
+                and dialog.screen.current_record
+                and self.sensible_widget != main.window
+                and self.ids):
+            dialog.screen.reload(self.ids, written=True)
 
     def end(self):
         super(WizardDialog, self).end()
